{# templates/pyproject.toml.jinja2 #}
# https://python-poetry.org/docs/
# https://packaging.python.org/en/latest/flow/#the-configuration-file

[tool.poetry]
name = "{{ package_name }}"
version = "0.0.1"
description = "A Lightweight Modular Autonomous True Multi Agent AI Framework."
authors = [
    "{{ author_name }} <{{ author_email }}>",
]
maintainers = [
    "{{ maintainer_name }} <{{ maintainer_email }}>",
]
readme = "README.md"
keywords = ["{{ package_name }}", "AGI", "Agentic AI", "ASI", "superagentx", "agent", "LLM"]

[tool.poetry.dependencies]
python = ">=3.12,<=3.13"
rich = "^13.9.2"
websockets = "^15.0.1"
superagentx = {extras = ["all"], version = "^1.0.0b5"}
superagentx-handlers = "^0.1.5.4"
fastapi = {extras = ["standard"], version = "^0.115.4"}
# opentelemetry-exporter-otlp-proto-grpc = "1.25.0"   # Temporally added for Chromadb issue https://github.com/chroma-core/chroma/issues/2571

[build-system]
requires = ["poetry-core"]
build-backend = "poetry.core.masonry.api"{{'\n'}}